import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Utils";

function GetDamage(damage)
	Turbine.PluginManager.LoadPlugin("DamageRead");
	local commands = Turbine.Shell.GetCommands();
	for k, v in pairs(commands) do
		if string.find(v, "damageread") == 1 then
			damage = tonumber(string.sub(v, 11));
		end
	end
	Turbine.PluginManager.UnloadScriptState("DamageRead");

	return damage;
end

DPSMeter = class(Turbine.UI.Window);

function DPSMeter:Constructor()
	Turbine.UI.Window.Constructor(self);

	self.player = Turbine.Gameplay.LocalPlayer();
	self.inCombat = self.player:IsInCombat();
	self.startCombat = Turbine.Engine.GetGameTime();
	self.endCombat = Turbine.Engine.GetGameTime();
	self.player.InCombatChanged = function(sender, args)
		self:combatChange();
	end
	self.currentDamage = GetDamage(0);
	self.startDamage = self.currentDamage;

	self.dragging = false;
	self.settings = DPSMeterLoadSettings();
	self:SetWantsUpdates(true);

	self:SetBackColor(Turbine.UI.Color( 1, 0, 0, 0));
	self:SetBackColorBlendMode(Turbine.UI.BlendMode.Normal);
	self:SetPosition(self.settings.x, self.settings.y);
	self:SetSize(80, 42);

	self.dps = Turbine.UI.Label();
	self.dps:SetParent(self);
	self.dps:SetPosition(1, 1);
	self.dps:SetSize(70, 20);
	self.dps:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.dps:SetFont(Turbine.UI.Lotro.Font.Verdana20);
	self.dps:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight);
	self.dps:SetZOrder(100);
	self.dps:SetMouseVisible(false);
	self.dps:SetText("---");

	self.timer = Turbine.UI.Label();
	self.timer:SetParent(self);
	self.timer:SetPosition(1, 21);
	self.timer:SetSize(70, 20);
	self.timer:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.timer:SetFont(Turbine.UI.Lotro.Font.Verdana20);
	self.timer:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight);
	self.timer:SetZOrder(100);
	self.timer:SetMouseVisible(false);
	self.timer:SetText("0.0");
end

function DPSMeter:UpdateVis()
	if self.settings.show == 1 then
		self:SetVisible(true);
	elseif self.settings.show == 2 then
		self:SetVisible(false);
	else
		if self.player:IsInCombat() then
			self:SetVisible(false);
		else
			self:SetVisible(true);
		end
	end
end

function DPSMeter:combatChange()
	self.inCombat = self.player:IsInCombat();
	if self.inCombat then
		self.startCombat = Turbine.Engine.GetGameTime();
		self.endCombat = Turbine.Engine.GetGameTime();
		self.startDamage = self.currentDamage;
		self:SetWantsUpdates(true);
	end
	self:UpdateVis();
	self:Update();
end

function DPSMeter:Update()
	local time = Turbine.Engine.GetGameTime();

	if self.player:IsInCombat() then
		self.endCombat = Turbine.Engine.GetGameTime();
	end

	if time < (self.endCombat + 2) then
		self.currentDamage = GetDamage(self.currentDamage);
	else
		self:SetWantsUpdates(false);
	end

	local delta = self.endCombat - self.startCombat;
	local min = math.floor(delta / 60);
	local sec = delta % 60;


	if min > 0 then
		if sec < 10 then
			self.timer:SetText(min .. ":0" .. string.format("%02.1f", sec));
		else
			self.timer:SetText(min .. ":" .. string.format("%02.1f", sec));
		end
		else
		self.timer:SetText(string.format("%02.1f", sec));
	end

	if delta > 0 then
		local dps = (self.currentDamage - self.startDamage) / delta;
		self.dps:SetText(string.format("%.1f", dps));
	else
		self.dps:SetText("---");
	end

end

function DPSMeter:SaveSettings()
	Turbine.PluginData.Save(Turbine.DataScope.Character, "DPSMeter", self.settings);
end

function DPSMeterLoadSettings()
	settings = Turbine.PluginData.Load(Turbine.DataScope.Character, "DPSMeter");
	if type(settings) ~= "table" then
		settings = { };
	end

	if not settings.x then
		settings.x = 0;
	end
	if not settings.y then
		settings.y = 0;
	end
	if not settings.show then
		settings.show = 0;
	end

	return settings;
end

function DPSMeter:MouseUp(args)
	if (args.Button == Turbine.UI.MouseButton.Left) then
		self:SaveSettings();
		self.dragging = false;
	end
end

function DPSMeter:MouseDown(args)
	if (args.Button == Turbine.UI.MouseButton.Left) then
		self.dragStartX = args.X;
		self.dragStartY = args.Y;
		self.dragging = true;
	end
end

function DPSMeter:MouseMove(args)
	if (self.dragging) then
		local x, y = self:GetPosition();
		x = x + (args.X - self.dragStartX);
		y = y + (args.Y - self.dragStartY);

		self:SetPosition(x, y);
		self.settings.x = x;
		self.settings.y = y;
	end
end

DPSMeterCommand = class(Turbine.ShellCommand);

function DPSMeterCommand:Execute(command, arguments)
	if arguments == "post" then
		DPSMeter.settings.show = 0;
		DPSMeter:SaveSettings();
		DPSMeter:UpdateVis();
	elseif arguments == "hide" then
		DPSMeter.settings.show = 2;
		DPSMeter:SaveSettings();
		DPSMeter:UpdateVis();
	elseif arguments == "show" then
		DPSMeter.settings.show = 1;
		DPSMeter:SaveSettings();
		DPSMeter:UpdateVis();
	else
		Turbine.Shell.WriteLine("Usage: /dpsmeter <post|show|hide>");
	end
end

function DPSMeterCommand:GetShortHelp()
	return "usage: /dpsmeter <post|show|hide>";
end

dpsMeterCommand = DPSMeterCommand();
Turbine.Shell.AddCommand('dpsmeter', dpsMeterCommand);
DPSMeter = DPSMeter();
DPSMeter:UpdateVis();